/*
PAELLOS - Plateforme d'Apprentissage En Ligne Libre Orientée Sémantique

Copyright (C) 2011 Observatoire de Paris

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformément aux dispositions de la Licence Publique Générale GNU, telle que publiée par la Free Software Foundation ; version 2 de la licence, ou encore (à votre choix) toute version ultérieure.

Ce programme est distribué dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de détail, voir la Licence Publique Générale GNU .

Vous devez avoir reçu un exemplaire de la Licence Publique Générale GNU en même temps que ce programme ; si ce n'est pas le cas, écrivez à la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

package paellos;

import java.io.*;
import java.net.URLEncoder;
import java.security.NoSuchAlgorithmException;
import java.sql.Blob;
import java.sql.SQLException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.regex.PatternSyntaxException;

import javax.servlet.*;
import javax.servlet.http.*;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.w3c.dom.Document;

import org.xml.sax.InputSource;

import paellos.jdb.*;
import paellos.base.*;

/**
 * Servlet pour CELA: lecture des réponses aux exercices
 *
 * @author Damien Guillaume
 */
public class Exercices extends HttpServlet {

    // valeurs par défaut, qu'on change dans le fichier ServletProperties
    protected String URLServeur;
    protected String pageLogin;
    protected String cheminSiteEtudiants;
    protected String cheminSiteTuteurs;
    protected String userDBURL;
    protected String userDBUSER;
    protected String userDBPASSWORD;
    protected String[] noms_formations;
    protected String[] titres_formations;
    
    UserDB userDB;
    ExerciceDB exoDB;
    
    Transformer reponsesXSLT;
    
    
    public void init(ServletConfig config) throws ServletException {
        super.init(config);
        Properties props = new Properties();
        try {
            props.load(new InputStreamReader(getClass().getResourceAsStream("ServletProperties"), "UTF-8"));
            URLServeur = props.getProperty("URLServeur");
            pageLogin = props.getProperty("pageLogin");
            cheminSiteEtudiants = props.getProperty("cheminSiteEtudiants");
            cheminSiteTuteurs = props.getProperty("cheminSiteTuteurs");
            userDBURL = props.getProperty("userDBURL");
            userDBUSER = props.getProperty("userDBUSER");
            userDBPASSWORD = props.getProperty("userDBPASSWORD");
            String s_formations = props.getProperty("formations");
            if (s_formations == null || "".equals(s_formations)) {
                noms_formations = null;
                titres_formations = null;
            } else {
                String[] formations = s_formations.split("\\|");
                noms_formations = new String[formations.length];
                titres_formations = new String[formations.length];
                for (int i=0; i<formations.length; i++) {
                    String[] nom_titre = formations[i].split(":");
                    noms_formations[i] = nom_titre[0];
                    if (nom_titre.length > 0)
                        titres_formations[i] = nom_titre[1];
                    else
                        titres_formations[i] = noms_formations[i];
                }
            }
        } catch (IOException ex) {
            log(Strings.get("erreur_proprietes") + " " + ex.getMessage());
        }
        userDB = new UserDB(userDBURL, userDBUSER, userDBPASSWORD);
        exoDB = new ExerciceDB(userDBURL, userDBUSER, userDBPASSWORD);
        try {
            if (!exoDB.tableExists()) {
                log(Strings.get("info_creation_table_exercices"));
                exoDB.createDB();
            }
        } catch (JDBException ex) {
            log("tableExists/createDB: " + ex.getMessage());
        }
        try {
            final TransformerFactory tFactory = TransformerFactory.newInstance() ;
            reponsesXSLT = tFactory.newTransformer(new StreamSource(getClass().getResourceAsStream("REPONSES.xsl")));
        } catch (Exception ex) {
            log("init Exercices: Création du transformer XSLT pour les réponses: " + ex.getClass().getName() + ": " + ex.getMessage());
        }
    }
    
    /**
     * Paramètres possibles pour une requête POST :
     * login : login pour la connexion au site
     * password : mot de passe pour la connexion au site
     *
     * numexercice : numéro de l'exercice à noter
     * note : note à donner à l'exercice
     */
    public void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        if (req.getCharacterEncoding() == null)
            req.setCharacterEncoding("UTF-8");
        if (req.getParameter("login") != null) {
            HttpSession session = req.getSession(true);
            String nom = req.getParameter("login");
            String password = req.getParameter("password");
            if(nom == null || nom.length() == 0)
                nom = "anonymous";
            
            // recherche de l'utilisateur dans la base
            User user = null;
            User finduser = new User(nom);
            finduser.valide = "1";
            try {
                Vector<User> v = userDB.findUsers(finduser);
                if (v.size() > 0)
                    user = v.get(0); // on prend le premier utilisateur avec le username
            } catch (JDBException ex) {
                log("JDBException: " + ex.getMessage());
            }
            if (user == null) {
                log(Strings.get("utilisateur_non_trouve") + " " + nom + " -> " + Strings.get("acces_refuse"));
                envoyerErreur(resp, Strings.get("acces_refuse"));
                return;
            }
            
            // test mot de passe
            try {
                if (password == null || !JDB.testEnc(password, user.password)) {
                    log(Strings.get("mauvais_mot_de_passe") + " " + nom);
                    envoyerErreur(resp, Strings.get("acces_refuse"));
                    return;
                }
            } catch (NoSuchAlgorithmException ex) {
                log("Exercices.doPost: NoSuchAlgorithmException: " + ex.getMessage());
                envoyerErreur(resp, Strings.get("acces_refuse"));
                return;
            }
            
            // obtention date d'aujourd'hui
            final Date now = new Date();
            final SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm");
            final String date = formatter.format(now);
            
            // test fermeture de compte
            if (user.datefermeture != null && !"".equals(user.datefermeture)) {
                Date fermeture;
                try {
                    fermeture = formatter.parse(user.datefermeture);
                    if (now.after(fermeture)) {
                        log(Strings.get("utilisation_compte_ferme") + " " + nom);
                        envoyerErreur(resp, Strings.get("acces_refuse_compte_ferme"));
                        return;
                    }
                } catch (ParseException ex) {
                    log("datefermeture ParseException: " + ex.getMessage());
                }
            }
            
            session.setAttribute("user", user);
            log(Strings.get("exercices_login") + " " + user.username);
            if (user.fonction.equals("tuteur") || user.fonction.equals("supertuteur") ||
                    user.fonction.equals("administrateur") || user.fonction.equals("système"))
                envoyerListeEtudiants(resp, user, null);
            else
                envoyerListeExercices(resp, user, null);
            return;
            
        } else if (req.getParameter("numexercice") != null) {
            final HttpSession session = req.getSession(false);
            if (session == null) {
                log(Strings.get("erreur_pas_de_session") + " " + req.getRequestURL());
                envoyerErreur(resp, Strings.get("session_expiree"));
                return;
            }
            final User user = (User)session.getAttribute("user");
            if (user == null) {
                log(Strings.get("erreur_user_null") + " " + req.getRequestURL());
                envoyerErreur(resp, Strings.get("erreur_donnees_session"));
                return;
            }
            if ("étudiant".equals(user.fonction)) {
                log(Strings.get("erreur_pas_un_tuteur") + " " + user.numero);
                envoyerErreur(resp, Strings.get("erreur_vous_pas_tuteur"));
                return;
            }
            final String numexercice = req.getParameter("numexercice");
            final String note = req.getParameter("note");
            final Exercice exo = chercherExercice(numexercice);
            if (exo == null) {
                log(Strings.get("erreur_no_exercice") + ": " + numexercice);
                envoyerErreur(resp, Strings.get("erreur_no_exercice"));
                return;
            }
            final User etudiant = chercherUser(exo.numuser);
            if (etudiant == null) {
                log("Exercices.doPost: numexercice: " + Strings.get("erreur_pas_etudiant_exo") + " " + numexercice);
                envoyerErreur(resp, Strings.get("erreur_no_exercice"));
                return;
            }
            if ("tuteur".equals(user.fonction) && !user.numero.equals(etudiant.numtuteur)) {
                log("Exercices.doPost: numexercice: " + Strings.get("erreur_exo_autre_tuteur") + " " + numexercice);
                envoyerErreur(resp, Strings.get("erreur_no_exercice"));
                return;
            }
            exo.note = note;
            try {
                exoDB.modExercice(exo);
            } catch (JDBException ex) {
                log("doPost: JDBException: " + ex.getMessage());
            }
            //envoyerPageExercice(resp, exo, etudiant, user);
            // pour garder l'URL et faciliter la copie de la réponse :
            resp.setStatus(HttpServletResponse.SC_MOVED_TEMPORARILY);
            resp.setHeader("Location", "Exercices?numexercice=" + numexercice);
            return;
        }
        envoyerErreur(resp, Strings.get("requete_non_comprise"));
    }
    
    /**
     * Paramètres possibles pour une requête GET :
     * numtuteur : numéro du tuteur (quand l'utilisateur est un tuteur, pour avoir sa liste d'étudiants)
     * numetudiant : numéro de l'étudiant dont on veut voir les exercices
     * numexercice : numéro de l'exercice à visualiser
     * imageexercice : numéro de l'exercice dont on veut obtenir l'image associée
     * imagetuteur : numéro de l'exercice dont on veut obtenir l'image associée (pour la réponse du tuteur)
     * tri : nom|date (nom par défaut)
     * format : brut pour obtenir une réponse d'exercice dans le format enregistré (XML ou texte simple)
     *          (aucune autre valeur n'est reconnue pour format)
     */
    public void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        final HttpSession session = req.getSession(false);
        if (session == null) {
            if (req.getQueryString() == null) {
                envoyerPageConnexion(req, resp);
                return;
            }
            log(Strings.get("erreur_pas_de_session") + " " + req.getRequestURL());
            String url = "Exercices?" + req.getQueryString();
            url = URLEncoder.encode(url, "UTF-8");
            envoyerErreur(resp, Strings.get("session_expiree_lien_1") + "<a href=\"" + pageLogin + "?redirection=" + url + "\">" +
                Strings.get("session_expiree_lien_2") + "</a> " + Strings.get("session_expiree_lien_3"));
            return;
        }
        final User user = (User)session.getAttribute("user");
        if (user == null) {
            if (req.getQueryString() == null) {
                envoyerPageConnexion(req, resp);
                return;
            }
            log(Strings.get("erreur_user_null") + " " + req.getRequestURL());
            String url = "Exercices?" + req.getQueryString();
            url = URLEncoder.encode(url, "UTF-8");
            envoyerErreur(resp, Strings.get("erreur_donnees_session") + " <a href=\"" + pageLogin + "?redirection=" + url + "\">" +
                Strings.get("revenir_identification") + "</a>.");
            return;
        }
        String numtuteur = req.getParameter("numtuteur");
        String numetudiant = req.getParameter("numetudiant");
        String numexercice = req.getParameter("numexercice");
        String imageexercice = req.getParameter("imageexercice");
        String imagetuteur = req.getParameter("imagetuteur");
        String tri = req.getParameter("tri");
        String format = req.getParameter("format");
        if (numtuteur == null && numetudiant == null && numexercice == null && imageexercice == null && imagetuteur == null) {
            if ("étudiant".equals(user.fonction))
                numetudiant = user.numero;
            else
                numtuteur = user.numero;
        }
        
        if (numtuteur != null) {
            if ("étudiant".equals(user.fonction)) {
                log("Exercices.doGet: " + Strings.get("erreur_pas_un_tuteur") + " " + user.numero);
                envoyerErreur(resp, Strings.get("erreur_vous_pas_tuteur"));
                return;
            }
            if (!user.numero.equals(numtuteur)) {
                log("Exercices.doGet: numtuteur: " + Strings.get("erreur_mauvais_no_tuteur") + " " + numtuteur);
                envoyerErreur(resp, Strings.get("erreur_mauvais_no_tuteur"));
                return;
            }
            envoyerListeEtudiants(resp, user, tri);
            
        } else if (numetudiant != null) {
            User etudiant = chercherUser(numetudiant);
            if (etudiant == null) {
                log("Exercices.doGet: numetudiant: " + Strings.get("erreur_mauvais_no_etudiant") + ": " + numetudiant);
                envoyerErreur(resp, Strings.get("erreur_mauvais_no_etudiant"));
                return;
            }
            if ("étudiant".equals(user.fonction)) {
                if (!user.numero.equals(numetudiant)) {
                    log("Exercices.doGet: numetudiant: " + Strings.get("erreur_pas_etudiant_connecte") + " " + numetudiant);
                    envoyerErreur(resp, Strings.get("erreur_mauvais_no_etudiant"));
                    return;
                }
                envoyerListeExercices(resp, user, null);
            } else {
                if ("tuteur".equals(user.fonction) && !etudiant.numtuteur.equals(user.numero)) {
                    log("Exercices.doGet: numetudiant: " + Strings.get("erreur_etudiant_autre_tuteur") + " " + numetudiant);
                    envoyerErreur(resp, Strings.get("erreur_mauvais_no_etudiant"));
                    return;
                }
                envoyerListeExercices(resp, etudiant, user);
            }
            
        } else if (numexercice != null) {
            Exercice exo = chercherExercice(numexercice);
            if (exo == null) {
                log("Exercices.doGet: numexercice: " + Strings.get("erreur_no_exercice") + " " + numexercice);
                envoyerErreur(resp, Strings.get("erreur_no_exercice"));
                return;
            }
            if ("étudiant".equals(user.fonction)) {
                if (!user.numero.equals(exo.numuser)) {
                    log("Exercices.doGet: numexercice: " + Strings.get("erreur_exo_autre_etudiant") + " " + numexercice);
                    envoyerErreur(resp, Strings.get("erreur_no_exercice"));
                    return;
                }
                if ("brut".equals(format))
                    envoyerExerciceBrut(resp, exo);
                else
                    envoyerPageExercice(resp, exo, user, null);
            } else {
                User etudiant = chercherUser(exo.numuser);
                if (etudiant == null) {
                    log("Exercices.doGet: numexercice: " + Strings.get("erreur_pas_etudiant_exo") + " " + numexercice);
                    envoyerErreur(resp, Strings.get("erreur_no_exercice"));
                    return;
                }
                if ("tuteur".equals(user.fonction) && !user.numero.equals(etudiant.numtuteur)) {
                    log("Exercices.doGet: numexercice: " + Strings.get("erreur_exo_autre_tuteur") + " " + numexercice);
                    envoyerErreur(resp, Strings.get("erreur_no_exercice"));
                    return;
                }
                if ("brut".equals(format))
                    envoyerExerciceBrut(resp, exo);
                else
                    envoyerPageExercice(resp, exo, etudiant, user);
            }
            
        } else if (imageexercice != null) {
            numexercice = imageexercice;
            final Exercice exo = chercherExercice(numexercice);
            if (exo == null) {
                log("Exercices.doGet: imageexercice: " + Strings.get("erreur_no_exercice") + " " + numexercice);
                envoyerErreur(resp, Strings.get("erreur_no_exercice"));
                return;
            }
            if ("étudiant".equals(user.fonction)) {
                if (!user.numero.equals(exo.numuser)) {
                    log("Exercices.doGet: imageexercice: " + Strings.get("erreur_exo_autre_etudiant") + " " + numexercice);
                    envoyerErreur(resp, Strings.get("erreur_no_exercice"));
                    return;
                }
            } else {
                final User etudiant = chercherUser(exo.numuser);
                if (etudiant == null) {
                    log("Exercices.doGet: imageexercice: " + Strings.get("erreur_pas_etudiant_exo") + " " + numexercice);
                    envoyerErreur(resp, Strings.get("erreur_no_exercice"));
                    return;
                }
                if ("tuteur".equals(user.fonction) && !user.numero.equals(etudiant.numtuteur)) {
                    log("Exercices.doGet: imageexercice: " + Strings.get("erreur_exo_autre_tuteur") + " " + numexercice);
                    envoyerErreur(resp, Strings.get("erreur_no_exercice"));
                    return;
                }
            }
            if (exo.nomimage == null || exo.image == null) {
                log("Exercices.doGet: imageexercice: " + Strings.get("erreur_aucune_image") + ": " + numexercice);
                envoyerErreur(resp, Strings.get("erreur_aucune_image"));
                return;
            } else {
                try {
                    envoyerImage(resp, exo.nomimage, exo.image.getBinaryStream());
                } catch (SQLException ex) {
                    log("Exercices.doGet: imageexercice: " + Strings.get("erreur") + ": SQLException: " + ex.getMessage());
                    envoyerErreur(resp, Strings.get("erreur_envoi_image"));
                    return;
                }
            }
        } else if (imagetuteur != null) {
            numexercice = imagetuteur;
            final Exercice exo = chercherExercice(numexercice);
            if (exo == null) {
                log("Exercices.doGet: imagetuteur: " + Strings.get("erreur_no_exercice") + " " + numexercice);
                envoyerErreur(resp, Strings.get("erreur_no_exercice"));
                return;
            }
            if ("étudiant".equals(user.fonction)) {
                if (!user.numero.equals(exo.numuser)) {
                    log("Exercices.doGet: imagetuteur: " + Strings.get("erreur_exo_autre_etudiant") + " " + numexercice);
                    envoyerErreur(resp, Strings.get("erreur_no_exercice"));
                    return;
                }
            } else {
                final User etudiant = chercherUser(exo.numuser);
                if (etudiant == null) {
                    log("Exercices.doGet: imagetuteur: " + Strings.get("erreur_pas_etudiant_exo") + " " + numexercice);
                    envoyerErreur(resp, Strings.get("erreur_no_exercice"));
                    return;
                }
                if ("tuteur".equals(user.fonction) && !user.numero.equals(etudiant.numtuteur)) {
                    log("Exercices.doGet: imagetuteur: " + Strings.get("erreur_exo_autre_tuteur") + " " + numexercice);
                    envoyerErreur(resp, Strings.get("erreur_no_exercice"));
                    return;
                }
            }
            if (exo.nomimagetuteur == null || exo.imagetuteur == null) {
                log("Exercices.doGet: imagetuteur: " + Strings.get("erreur_aucune_image") + ": " + numexercice);
                envoyerErreur(resp, Strings.get("erreur_aucune_image"));
                return;
            } else {
                try {
                    envoyerImage(resp, exo.nomimagetuteur, exo.imagetuteur.getBinaryStream());
                } catch (SQLException ex) {
                    log("Exercices.doGet: imagetuteur: " + Strings.get("erreur") + ": SQLException: " + ex.getMessage());
                    envoyerErreur(resp, Strings.get("erreur_envoi_image"));
                    return;
                }
            }
        }
    }
    
    public User chercherUser(final String numuser) {
        User finduser = new User();
        finduser.numero = numuser;
        finduser.valide = "1";
        try {
            Vector<User> v = userDB.findUsers(finduser);
            if (v.size() != 0) {
                User tuteur = v.get(0);
                return(tuteur);
            }
        } catch (JDBException ex) {
            log("chercherUser: JDBException: " + ex.getMessage());
        }
        log("chercherUser: pas d'utilisateur " + numuser);
        return(null);
    }
    
    public Exercice chercherExercice(final String numero) {
        Exercice findexo = new Exercice();
        findexo.numero = numero;
        try {
            Vector<Exercice> v = exoDB.findExercices(findexo);
            if (v.size() != 0)
                return(v.get(0));
        } catch (JDBException ex) {
            log("chercherExercice: JDBException: " + ex.getMessage());
        }
        log("chercherExercice: pas d'exercice avec numero=" + numero);
        return(null);
    }
    
    public void envoyerErreur(final HttpServletResponse resp, final String errmsg) {
        try {
            resp.setHeader("Pragma", "no-cache");
            resp.setHeader("Cache-Control", "no-cache");
            resp.setHeader("Expires", "0");
            resp.setContentType("text/html; charset=UTF-8");
            
            final PrintWriter pw = resp.getWriter();
            pw.println("<html>");
            pw.println("<head>");
            pw.println("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">");
            pw.println("<title>" + Strings.get("titre_erreur") + "</title>");
            pw.println(" <link href=\"paellos.css\" rel=\"stylesheet\" type=\"text/css\">");
            pw.println("</head>");
            pw.println("<body>");
            pw.println(Strings.get("une_erreur") + " " + errmsg);
            pw.println("<br>");
            pw.println(Strings.get("bouton_retour"));
            pw.println("</body>");
            pw.println("</html>");
            pw.flush();
        } catch (IOException ex) {
            log("envoyerErreur: " + ex.getMessage());
        }
    }
    
    public void envoyerPageConnexion(final HttpServletRequest req, final HttpServletResponse resp) {
        try {
            getServletContext().getRequestDispatcher("/WEB-INF/jsp/exercices.jsp").forward(req, resp);
        } catch (ServletException ex) {
            log("Exercices: envoyerPageConnexion: " + ex.getClass() + ": " + ex.getMessage());
            envoyerErreur(resp, Strings.get("erreur_cf_log"));
        } catch (IOException ex) {
            log("Exercices: envoyerPageConnexion: " + ex.getClass() + ": " + ex.getMessage());
            envoyerErreur(resp, Strings.get("erreur_cf_log"));
        }
    }
    
    public void envoyerListeEtudiants(final HttpServletResponse resp, final User tuteur, final String tri) {
        try {
            resp.setHeader("Pragma", "no-cache");
            resp.setHeader("Cache-Control", "no-cache");
            resp.setHeader("Expires", "0");
            resp.setContentType("text/html; charset=UTF-8");
            final Writer out = resp.getWriter();
            final PrintWriter pw = new PrintWriter(out);
            pw.println("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">");
            pw.println("<html>");
            pw.println("<head>");
            pw.println(" <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">");
            pw.println(" <title>" + Strings.get("liste_etudiants") + "</title>");
            pw.println(" <link href=\"paellos.css\" rel=\"stylesheet\" type=\"text/css\">");
            pw.println("</head>");
            pw.println("<body>");
            if ("tuteur".equals(tuteur.fonction))
                pw.println("<h2>" + Strings.get("etudiants_de") + " " + tuteur.prenom + " " + tuteur.nom + "</h2>");
            else
                pw.println("<h2>" + Strings.get("titre_etudiants") + "</h2>");
            pw.println("<table border=\"1\">");
            pw.println(" <tr><th><a href=\"Exercices?tri=nom\">" + Strings.get("titre_nom") + "</a></th>" +
                "<th><a href=\"Exercices?tri=date\">" + Strings.get("dernier_exercice") + "</a></th></tr>");
            final User finduser = new User();
            finduser.fonction = "étudiant";
            if ("tuteur".equals(tuteur.fonction))
                finduser.numtuteur = tuteur.numero;
            // cas "supertuteur" et "système" : tous les étudiants
            finduser.valide = "1";
            if (noms_formations != null && noms_formations.length == 1)
                finduser.formation = noms_formations[0];
            final SimpleDateFormat frenchformatter = new SimpleDateFormat(Strings.get("format_date"));
            final SimpleDateFormat isoformatter = new SimpleDateFormat("yyyy-MM-dd");
            final String maintenant = isoformatter.format(new Date());
            // obtention de la liste des étudiants pertinents et des dates de dernier exo
            final Vector<User> etudiants = new Vector<User>();
            final Vector<Date> dates = new Vector<Date>();
            try {
                final Vector<User> v = userDB.findUsers(finduser);
                for (User etudiant : v) {
                    boolean formationok;
                    if (noms_formations == null || etudiant.formation == null)
                        formationok = true;
                    else
                        formationok = Arrays.asList(noms_formations).contains(etudiant.formation);
                    final boolean dateok = (etudiant.datefermeture == null || "".equals(etudiant.datefermeture) ||
                        etudiant.datefermeture.compareTo(maintenant) > 0);
                    if (formationok && dateok)
                        etudiants.add(etudiant);
                }
                if (!"date".equals(tri))
                    Collections.sort(etudiants); // tri par nom par défaut
                for (User etudiant : etudiants) {
                    String sdate = exoDB.dateDernierExercice(etudiant.numero);
                    if (sdate == null)
                        dates.add(null);
                    else {
                        try {
                            dates.add(isoformatter.parse(sdate));
                        } catch (ParseException ex) {
                            log("envoyerListeEtudiants: date: ParseException: " + ex.getMessage());
                            dates.add(null);
                        }
                    }
                }
            } catch (JDBException ex) {
                log("JDBException: " + ex.getMessage());
            }
            // tri par date
            if ("date".equals(tri)) {
                for (int i=0; i<dates.size(); i++)
                    for (int j=i+1; j<dates.size(); j++) {
                        Date datei = dates.get(i);
                        Date datej = dates.get(j);
                        if ((datei == null && datej != null) || ((datei != null && datej != null) && datei.compareTo(datej) < 0)) {
                            final User tempUser = etudiants.get(i);
                            etudiants.setElementAt(etudiants.get(j), i);
                            etudiants.setElementAt(tempUser, j);
                            final Date tempDate = dates.get(i);
                            dates.setElementAt(dates.get(j), i);
                            dates.setElementAt(tempDate, j);
                        }
                    }
            }
            // affichage
            for (int i=0; i<etudiants.size(); i++) {
                int ntab = (i%2==0 ? 1 : 2);
                final User etudiant = etudiants.get(i);
                final Date dernier = dates.get(i);
                pw.println(" <tr>");
                pw.println("  <td class=\"t" + ntab + "\"><a href=\"Exercices?numetudiant=" + etudiant.numero + "\">" +
                    etudiant.prenom + " " + etudiant.nom + "</a></td>");
                String sdate;
                if (dernier == null)
                    sdate = "-";
                else
                    sdate = frenchformatter.format(dernier);
                pw.println("  <td class=\"t" + ntab + "\">" + sdate + "</td>");
                pw.println(" </tr>");
            }
            pw.println("</table>");
            pw.println("<p><a href=\"" + cheminSiteTuteurs + "index.html\">" + Strings.get("revenir_accueil") + "</a></p>");
            pw.println("</body>");
            pw.println("</html>");
            pw.flush();
        } catch (IOException ex) {
            log("envoyerListeEtudiants: " + ex.getMessage());
        }
    }
    
    public void envoyerListeExercices(final HttpServletResponse resp, final User etudiant, final User tuteur) {
        try {
            // pas de cache parce-que les notes peuvent changer et il faut que la page soit à jour
            resp.setHeader("Pragma", "no-cache");
            resp.setHeader("Cache-Control", "no-cache");
            resp.setHeader("Expires", "0");
            resp.setContentType("text/html; charset=UTF-8");
            
            final Writer out = resp.getWriter();
            final PrintWriter pw = new PrintWriter(out);
            pw.println("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">");
            pw.println("<html>");
            pw.println("<head>");
            pw.println(" <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">");
            pw.println(" <title>" + Strings.get("reponses_exos") + "</title>");
            pw.println(" <link href=\"paellos.css\" rel=\"stylesheet\" type=\"text/css\">");
            pw.println("</head>");
            pw.println("<body>");
            pw.println("<h2>" + Strings.get("exercices_de") + " " + etudiant.prenom + " " + etudiant.nom + "</h2>");
            pw.println("<p>" + Strings.get("info_titre_exo") + "</p>");
            pw.println("<p>" + Strings.get("info_lien_chemin") + "</p>");
            pw.println("<table border=\"1\">");
            pw.print(" <tr><th></th><th>" + Strings.get("titre_titre") + "</th><th>" + Strings.get("titre_chemin") +
                "</th><th>" + Strings.get("titre_date") + "</th>");
            if (tuteur != null)
                pw.print("<th>" + Strings.get("titre_note") + "</th>");
            pw.println("</tr>");
            final Exercice findexo = new Exercice();
            findexo.numuser = etudiant.numero;
            String cheminSite;
            if (tuteur != null)
                cheminSite = cheminSiteTuteurs;
            else
                cheminSite = cheminSiteEtudiants;
            try {
                int no = 0;
                final Vector<Exercice> v = exoDB.findExercices(findexo);
                try { // tri par numero
                    for (int i=0; i<v.size(); i++) {
                        for (int j=i+1; j<v.size(); j++) {
                            int numero_i = Integer.parseInt(v.get(i).numero);
                            int numero_j = Integer.parseInt(v.get(j).numero);
                            if (numero_j < numero_i) {
                                final Exercice tempExercice = v.get(i);
                                v.setElementAt(v.get(j), i);
                                v.setElementAt(tempExercice, j);
                            }
                        }
                    }
                } catch (NumberFormatException ex) {
                    log("envoyerListeExercices: NumberFormatException: " + ex.getMessage());
                }
                for (int i=0; i<v.size(); i++) {
                    int ntab = (i%2==0 ? 1 : 2);
                    Exercice exo = v.get(i);
                    pw.println(" <tr>");
                    String chemintitre = exo.chemin + ":" + exo.titre;
                    boolean dejavu = false;
                    for (int j=0; j<i; j++) {
                        Exercice exo2 = v.get(j);
                        String chemintitre2 = exo2.chemin + ":" + exo2.titre;
                        if (chemintitre.equals(chemintitre2)) {
                            dejavu = true;
                            break;
                        }
                    }
                    if (!dejavu) {
                        no++;
                        pw.println("  <td class=\"t" + ntab + "\">"+no+"</td>");
                    } else
                        pw.println("  <td class=\"t" + ntab + "\"></td>");
                    String titre;
                    if (exo.titre != null && !"".equals(exo.titre))
                        titre = exo.titre;
                    else
                        titre = Strings.get("sans_titre");
                    pw.println("  <td class=\"t" + ntab + "\"><a href=\"Exercices?numexercice="+exo.numero+"\">" +
                        titre + "</a></td>");
                    pw.println("  <td class=\"t" + ntab + "\"><a href=\"" + URLServeur + cheminSite + exo.chemin+"\">" + exo.chemin + "</a></td>");
                    pw.print("  <td class=\"t" + ntab + "\">");
                    if (exo.date != null) {
                        final SimpleDateFormat frenchformatter = new SimpleDateFormat(Strings.get("format_date"));
                        final SimpleDateFormat isoformatter = new SimpleDateFormat("yyyy-MM-dd");
                        try {
                            pw.print(frenchformatter.format(isoformatter.parse(exo.date)));
                        } catch (ParseException ex) {
                            log("envoyerListeExercices: date: ParseException: " + ex.getMessage());
                        }
                    }
                    pw.println("</td>");
                    if (tuteur != null) {
                        String note = exo.note;
                        if (note == null)
                            note = "";
                        pw.print("<td class=\"t" + ntab + "\">" + note + "</td>");
                    }
                    pw.println(" </tr>");
                }
            } catch (JDBException ex) {
                log("JDBException: " + ex.getMessage());
            }
            pw.println("</table>");
            if (tuteur != null)
                pw.println("<p><a href=\"Exercices?numtuteur=" + tuteur.numero + "\">" + Strings.get("revenir_liste_etudiants") + "</a></p>");
            pw.println("<p><a href=\"" + cheminSite + "index.html\">" + Strings.get("revenir_accueil") + "</a></p>");
            pw.println("</body>");
            pw.println("</html>");
            pw.flush();
        } catch (IOException ex) {
            log("envoyerListeExercices: " + ex.getMessage());
        }
    }
    
    private static boolean fichierImage(final String nomimage) {
        if (nomimage == null)
            return(true);
        final String lf = nomimage.toLowerCase();
        return(lf.endsWith(".png") || lf.endsWith(".gif") || lf.endsWith(".jpg") || lf.endsWith(".jpeg"));
    }
    
    public void envoyerExerciceBrut(final HttpServletResponse resp, final Exercice exo) {
        try {
            resp.setHeader("Pragma", "no-cache");
            resp.setHeader("Cache-Control", "no-cache");
            resp.setHeader("Expires", "0");
            if (exo.reponse.startsWith("<?xml"))
                resp.setContentType("text/xml; charset=UTF-8");
            else
                resp.setContentType("text/plain; charset=UTF-8");
            final Writer out = resp.getWriter();
            final PrintWriter pw = new PrintWriter(out);
            pw.print(exo.reponse);
            pw.close();
        } catch (IOException ex) {
            log("envoyerExerciceBrut: " + ex.getMessage());
        }
    }
    
    public void envoyerPageExercice(final HttpServletResponse resp, final Exercice exo, final User etudiant, final User tuteur) {
        try {
            resp.setHeader("Pragma", "no-cache");
            resp.setHeader("Cache-Control", "no-cache");
            resp.setHeader("Expires", "0");
            resp.setContentType("text/html; charset=UTF-8");
            final Writer out = resp.getWriter();
            final PrintWriter pw = new PrintWriter(out);
            pw.println("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">");
            pw.println("<html>");
            pw.println("<head>");
            pw.println(" <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">");
            pw.println(" <title>" + Strings.get("reponse_exo") + "</title>");
            pw.println(" <link href=\"paellos.css\" rel=\"stylesheet\" type=\"text/css\">");
            String rephtml = exo.reponse;
            boolean xml = (rephtml != null && rephtml.startsWith("<?xml") && rephtml.indexOf("<REPONSES>") != -1);
            if (xml) {
                pw.println(" <style type=\"text/css\">");
                pw.println("   @font-face { ");
                pw.println("     font-family: 'STIXSubset-Regular';");
                pw.println("     src: url('STIXSubset-Regular.eot');");
                pw.println("     src: local('STIXSubset-Regular'), url('STIXSubset-Regular.ttf') format('truetype');");
                pw.println("   }");
                pw.println("   span.symbole { font-family: STIXSubset-Regular, serif } ");
                pw.println(" </style>");
            }
            if (tuteur != null && exo.reponsetuteur == null) {
                /*
                pw.println("<script type=\"text/javascript\" language=\"JavaScript\">");
                pw.println("<!--");
                pw.println("    function verif(nomj, nomf) {");
                pw.println("        if (document.applets[nomj].getEditing())");
                pw.println("            setTimeout('verif(\"' + nomj + '\",\"' + nomf + '\")', 500);");
                pw.println("        else {");
                pw.println("            var texte = document.applets[nomj].getTexte();");
                pw.println("            if (texte != null)");
                pw.println("                document.forms[nomf].reponse.value = texte;");
                pw.println("        }");
                pw.println("    }");
                pw.println("    ");
                pw.println("    function editer(nomj, nomf) {");
                pw.println("        document.forms[nomf].reponse.readOnly = true;");
                pw.println("        var texte = document.forms[nomf].reponse.value;");
                pw.println("        if (texte != '' && texte.indexOf('<?xml') == -1)");
                pw.println("            texte = '<?xml version=\"1.0\" encoding=\"UTF-8\"?>\\n<REPONSES>\\n' + texte + '\\n<\\/REPONSES>\\n';");
                pw.println("        document.applets[nomj].editer(texte);");
                pw.println("        setTimeout('verif(\"' + nomj + '\",\"' + nomf + '\")', 500);");
                pw.println("    }");
                pw.println("    function copier() {");
                pw.println("        var xmlhttp = new XMLHttpRequest();");
                pw.println("        xmlhttp.open('GET', document.URL + '&format=brut', false);");
                pw.println("        xmlhttp.send();");
                pw.println("        document.forms['formulaire1'].reponse.value = xmlhttp.responseText;");
                pw.println("    }");
                pw.println("    //-->");
                pw.println("</script>");
                */
                pw.println("<script type=\"text/javascript\" src=\"editeur/ckeditor.js\"></script>");
                pw.println("<script type=\"text/javascript\" src=\"editeur/editeur.js\"></script>");
                pw.println("<script type=\"text/javascript\">");
                pw.println("    function copier() {");
                pw.println("        var xmlhttp = new XMLHttpRequest();");
                pw.println("        xmlhttp.open('GET', document.URL + '&format=brut', false);");
                pw.println("        xmlhttp.send();");
                pw.println("        CKEDITOR.instances['ckeditor1'].setData(xmlhttp.responseText);");
                pw.println("    }");
                pw.println("    editeur_reponses.serveur_equations = 'CetteServlet?equation=';");
                pw.println("</script>");
            }
            pw.println("</head>");
            pw.println("<body>");
            if (exo.titre != null && !"".equals(exo.titre))
                pw.println("<h2>" + exo.titre + "</h2>");
            if (tuteur != null)
                pw.println("<p>" + Strings.get("nom_etudiant") + " " + etudiant.prenom + " " + etudiant.nom + "</p>");
            String cheminSite;
            if (tuteur != null)
                cheminSite = cheminSiteTuteurs;
            else
                cheminSite = cheminSiteEtudiants;
            pw.println("<p>" + Strings.get("titre_chemin") + " : <a href=\"" + URLServeur + cheminSite + exo.chemin+"\">" + exo.chemin + "</a></p>");
            final SimpleDateFormat frenchformatter = new SimpleDateFormat(Strings.get("format_date"));
            final SimpleDateFormat isoformatter = new SimpleDateFormat("yyyy-MM-dd");
            try {
                pw.println("<p>" + Strings.get("titre_date") + " : " + frenchformatter.format(isoformatter.parse(exo.date)) + "</p>");
            } catch (ParseException ex) {
                log("envoyerPageExercice: date: ParseException: " + ex.getMessage());
            }
            if (xml) {
                final Document doc = lectureDocumentXML(rephtml);
                if (doc == null)
                    xml = false;
                else
                    rephtml = xmlVersHTMLbody(doc);
            }
            if (!xml)
                rephtml = texteVersHTMLbody(rephtml);
            pw.println(rephtml);
            try {
                if (exo.image != null && exo.image.length() > 0) {
                    if (fichierImage(exo.nomimage)) {
                        pw.println("<p>" + Strings.get("image") + "</p>");
                        pw.println("<p><img src=\"Exercices?imageexercice=" + exo.numero + "\" alt=\"" + exo.nomimage + "\"></p>");
                    } else {
                        pw.println("<p>" + Strings.get("fichier"));
                        pw.println("<a href=\"Exercices?imageexercice=" + exo.numero + "\">" + exo.nomimage + "</a></p>");
                    }
                }
            } catch (SQLException ex) {
                log("envoyerPageExercice: SQLException: " + ex.getMessage());
            }
            
            if (tuteur != null && exo.reponsetuteur != null) {
                // formulaire pour la note
                // si le tuteur n'a pas encore envoyé sa réponse, le champ est dans le formulaire de réponse
                pw.println("<form method=\"post\" action=\"Exercices\">");
                pw.print("  <p>" + Strings.get("titre_note") + " : <input type=\"text\" name=\"note\" size=\"10\" maxlength=\"10\"");
                if (exo.note != null)
                    pw.print(" value=\"" + exo.note + "\"");
                pw.println(">");
                pw.println("  <input type=\"hidden\" name=\"numexercice\" value=\"" + exo.numero + "\">");
                pw.println("  <input type=\"submit\" value=\"" + Strings.get("enregistrer") + "\"></p>");
                pw.println("</form>");
            }
            if (tuteur != null || exo.reponsetuteur != null) {
                pw.println("<hr>");
            }
            if (tuteur != null && exo.reponsetuteur == null) {
                // formulaire de réponse du tuteur
                pw.println("<form action=\"CetteServlet\" enctype=\"multipart/form-data\" method=\"post\">");
                pw.print("  <p>" + Strings.get("titre_note") + " : <input type=\"text\" name=\"note\" size=\"10\" maxlength=\"10\"");
                if (exo.note != null)
                    pw.print(" value=\"" + exo.note + "\"");
                pw.println(">");
                pw.println("<p>" + Strings.get("reponse") + " ");
                pw.println("<input onclick=\"copier()\" value=\"" + Strings.get("copier_reponse") + "\" type=\"button\"></p>");
                pw.println("<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\" style=\"width: 100%\">");
                pw.println("<tr>");
                pw.print("<td>");
                pw.print("<textarea id=\"ckeditor1\" name=\"reponse\" rows=\"10\" cols=\"70\"></textarea>");
                pw.print("<input name=\"numexercice\" value=\"" + exo.numero + "\" type=\"hidden\">");
                //pw.print("<applet height=\"10\" width=\"10\" code=\"jaxeapplet.JaxeApplet\" archive=\"jaxe/AppletJaxePAELLOS.jar\" name=\"jaxe1\">");
                //pw.print("<param value=\"jaxe/config/REPONSES_config.xml\" name=\"config\">" + Strings.get("java_doit_etre_active") + "</applet>");
                //pw.print("<input onclick=\"editer('jaxe1','formulaire1')\" value=\"" + Strings.get("editeur") + "\" type=\"button\">");
                pw.println("</td>");
                pw.println("</tr>");
                pw.println("<tr>");
                pw.println("<td>" + Strings.get("image") + " <input name=\"image\" type=\"file\">");
                pw.println("<br>");
                pw.println("<small>" + Strings.get("conditions_image") + "</small></td>");
                pw.println("</tr>");
                pw.println("<tr>");
                pw.println("<td align=\"center\"><input value=\"" + Strings.get("envoyer") + "\" name=\"Submit\" type=\"submit\"></td>");
                pw.println("</tr>");
                pw.println("</table>");
                pw.println("</form>");
            } else if (exo.reponsetuteur != null) {
                // affichage de la réponse du tuteur
                pw.println("<p>" + Strings.get("reponse") + "</p>");
                String reptuteur = exo.reponsetuteur;
                boolean reptuteurxml = (reptuteur != null && reptuteur.startsWith("<?xml") && reptuteur.indexOf("<REPONSES>") != -1);
                if (reptuteurxml) {
                    final Document doc = lectureDocumentXML(reptuteur);
                    if (doc == null)
                        reptuteurxml = false;
                    else
                        reptuteur = xmlVersHTMLbody(doc);
                }
                if (!reptuteurxml)
                    reptuteur = texteVersHTMLbody(reptuteur);
                pw.println(reptuteur);
                try {
                    if (exo.imagetuteur != null && exo.imagetuteur.length() > 0) {
                        if (fichierImage(exo.nomimagetuteur)) {
                            pw.println("<p>" + Strings.get("image") + "</p>");
                            pw.println("<p><img src=\"Exercices?imagetuteur=" + exo.numero + "\"></p>");
                        } else {
                            pw.println("<p>" + Strings.get("fichier"));
                            pw.println("<a href=\"Exercices?imagetuteur=" + exo.numero + "\">" + exo.nomimagetuteur + "</a></p>");
                        }
                    }
                } catch (SQLException ex) {
                    log("envoyerPageExercice: SQLException: " + ex.getMessage());
                }
            }
            if (tuteur != null || exo.reponsetuteur != null)
                pw.println("<hr>");
            
            pw.println("<p><a href=\"Exercices?numetudiant=" + exo.numuser + "\">" + Strings.get("revenir_liste_exos") + "</a></p>");
            pw.println("</body>");
            pw.println("</html>");
            pw.flush();
        } catch (IOException ex) {
            log("envoyerPageExercice: " + ex.getMessage());
        }
    }
    
    protected Document lectureDocumentXML(final String texte) {
        Document doc = null;
        try {
            final DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            final DocumentBuilder docbuilder = dbf.newDocumentBuilder();
            doc = docbuilder.parse(new InputSource(new StringReader(texte)));
        } catch (final Exception ex) {
            log("envoyerPageExercice: " + ex.getClass().getName() + ": " + ex.getMessage());
        }
        return(doc);
    }
    
    protected static String texteVersHTMLbody(final String texte) {
        String rephtml = texte;
        for (int i=0; i<rephtml.length(); i++) {
            char c = rephtml.charAt(i);
            if (c == '\n' || c == '\r') {
                int i2 = i+1;
                if (c == '\r' && i2 < rephtml.length() && rephtml.charAt(i2) == '\n')
                    i2++;
                rephtml = rephtml.substring(0, i) + "</p>\n<p>" + rephtml.substring(i2);
                i += 7;
            } else if (c == '&') {
                rephtml = rephtml.substring(0, i) + "&amp;" + rephtml.substring(i+1);
                i += 4;
            } else if (c == '<') {
                rephtml = rephtml.substring(0, i) + "&lt;" + rephtml.substring(i+1);
                i += 3;
            } else if (c == '>') {
                rephtml = rephtml.substring(0, i) + "&gt;" + rephtml.substring(i+1);
                i += 3;
            }
        }
        rephtml = "<p>" + rephtml + "</p>";
        return(rephtml);
    }
    
    protected String xmlVersHTMLbody(final Document doc) {
        final StringWriter sw = new StringWriter();
        try {
            reponsesXSLT.transform(new DOMSource(doc), new StreamResult(sw));
        } catch (TransformerException ex) {
            log("envoyerPageExercice: " + ex.getClass().getName() + ": " + ex.getMessage());
        }
        String rephtml = sw.toString();
        
        // correction des erreurs de xalan/xsltc
        // cf http://stackoverflow.com/questions/11952289/serializing-supplementary-unicode-characters-into-xml-documents-with-java
        // on va transformer "&#55349;&#56476;" (invalide) en "&#119964;"
        // ici on ne gère que les caractères qui commencent par 0xD835 en UTF-16 (utilisés par les symboles),
        // mais on pourrait généraliser à l'ensemble 0xD800..0xDBFF
        try {
            int pos = 0;
            int ind = rephtml.indexOf("&#55349;&#", pos);
            String sub, s, ent;
            int v1, v2, cp;
            byte[] tb = new byte[4];
            while (ind != -1) {
                sub = rephtml.substring(ind + 10, ind + 16);
                if (sub.matches("\\d{5};")) {
                    v1 = 55349;
                    v2 = Integer.parseInt(sub.substring(0, 5));
                    tb[0] = (byte)((v1 >> 8) & 0xFF);
                    tb[1] = (byte)(v1 & 0xFF);
                    tb[2] = (byte)((v2 >> 8) & 0xFF);
                    tb[3] = (byte)(v2 & 0xFF);
                    s = new String(tb, "UTF-16");
                    cp = s.codePointAt(0);
                    ent = "&#" + String.valueOf(cp) + ";";
                    rephtml = rephtml.substring(0, ind) + ent + rephtml.substring(ind + 16);
                    pos += ent.length();
                } else
                    pos += 8;
                ind = rephtml.indexOf("&#55349;&#", pos);
            }
        } catch (NumberFormatException ex) {
            log("xmlVersHTMLbody: NumberFormatException:" + ex.getMessage());
        } catch (UnsupportedEncodingException ex) {
            log("xmlVersHTMLbody: UnsupportedEncodingException:" + ex.getMessage());
        }
        
        rephtml = rephtml.replace("<html>", "");
        rephtml = rephtml.replace("</html>", "");
        rephtml = rephtml.replace("<body>", "");
        rephtml = rephtml.replace("</body>", "");
        final int pdh = rephtml.indexOf("<head>");
        final int pfh = rephtml.indexOf("</head>");
        if (pdh != -1 && pfh != -1)
            rephtml = rephtml.substring(0, pdh) + rephtml.substring(pfh + 7);
        return(rephtml);
    }
    
    public void envoyerImage(final HttpServletResponse resp, final String nomimage, final InputStream in) {
        try {
            final ServletContext contexte = getServletContext();
            if (contexte == null)
                throw new IOException("envoyerImage: getServletContext est null");
            final String mime = contexte.getMimeType(nomimage.toLowerCase());
            if (mime == null)
                throw new IOException("envoyerImage: " + Strings.get("mime_inconnu") + " \"" + nomimage + "\"");
            resp.setContentType(mime);
            if (!fichierImage(nomimage))
                resp.setHeader("Content-disposition", "attachment; filename=\"" + nomimage + "\"");
            
            final OutputStream out = resp.getOutputStream();
            
            final BufferedInputStream bin = new BufferedInputStream(in);
            final BufferedOutputStream bout = new BufferedOutputStream(out);
            
            int length = 32;
            byte[] ch = new byte[length]; 
            while((length = bin.read(ch)) != -1)
                bout.write(ch, 0, length); 
            
            bin.close();
            bout.flush();
        } catch (IOException ex) {
            log("envoyerImage: IOException: " + ex.getMessage());
        }
    }
    
}

